# SEDModS -- VLKB SED Models web service

This service is meant to provide filtering on top of the SED Models
(big, 30M rows) catalogue table. The table was initially served as a
RDBMS table and queried through SQL directly.

This work moves it onto a python+HDF5 solution.

The service is deployed as a python+WSGI on httpd, within a podman
container to help maintenance.

## Deploy SEDModS

### Project content description
The `Dockerfile` uses::
 - an httpd server (running on container's port 80)
 - python and WSGI packages
It needs a properly configured `wsgi.conf` that will be loaded within
the `http/conf.d` of the container.

### Build and run the container image
To build the container image:

    podman build --tag vlkb-sedmods -f ./Dockerfile

It is suggested to have a dedicated user to run this in production.

Once the podman image is ready and the two directories are in place, run:

    podman run -dt \
        --name sedmod-test \
        --rm \
        -v $PWD/sed-data:/sed-data:z \
        -v $PWD/wsgi-scripts:/var/www/wsgi-scripts:z \
        -p 8080:80 vlkb-sedmods

where the name is optional and of your choice and the left-hand side of
the _:_ in the _-v_ arguments must point to the actual location of the
two folders to be mounted in the container.

In the example the _-v_ arguments are considered such as the command is
run from the local working copy and the HDF5 SED Models file is actually
within the `sed-data` folder.

Also, _-p_ maps the 80 port of the container onto the 8080 port on the
host server, this must be changed if the host's 8080 is already in use.

### Service endpoints

The service implementation is descripbed [here](README_implementation.md).

Then the service presents the _/searchd_ endpoint. Arguments are separated
by underscore and their meaning is as in this C++ code snipped:

```cpp
 QString args = QString("'%1_%2_%3_%4_%5_%6_%7_0_%8_%9'")
                           .arg(sedFitInputW)
                           .arg(sedFitInputF)
                           .arg(sedFitInputErrF)
                           .arg(sedFitInputFflag)
                           .arg(ui->distTheoLineEdit->text())
                           .arg(ui->prefilterLineEdit->text())
                           .arg(sedWeights)
                           .arg(sedFitInputW)
                           .arg(ui->delta_chi2_lineEdit->text());
```

Access the service by: 

    curl localhost:8080/searchd/?arg1_arg2_...-_arg9 > output.json

Response is in JSON format.

### Network Proxy
The service can be made visible on a specific context path in the host
server's http using the httpd _ProxyPass_ directive, like
  
    <Location "/sedmods">
        ProxyPass "http://localhost:8080"
    </Location>

where _/sedmods_ is an example and the _8080_ port depends on the passed
parameters to the podman run command (see above).
